"""steps DB change: add created_at index

Revision ID: db41106b9f1a
Revises: a5feab7712fe
Create Date: 2024-12-02 16:09:57.679626+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "db41106b9f1a"
down_revision: Union[str, None] = "a5feab7712fe"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("created_at_org_index", "steps", ["created_at", "organization_id"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("created_at_org_index", table_name="steps")
    # ### end Alembic commands ###
