# This file was auto-generated by Fern from our API Definition.

import typing

RunStatus = typing.Union[
    typing.Literal["created", "queued", "running", "timed_out", "failed", "terminated", "completed", "canceled"],
    typing.Any,
]
