"""Add task prompt template, complete criterion and complete criterion

Revision ID: 2d79d5fc1baa
Revises: 1909715536dc
Create Date: 2024-11-21 07:08:19.177274+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "2d79d5fc1baa"
down_revision: Union[str, None] = "1909715536dc"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("tasks", sa.Column("prompt_template", sa.String(), nullable=True))
    op.add_column("tasks", sa.Column("complete_criterion", sa.String(), nullable=True))
    op.add_column("tasks", sa.Column("terminate_criterion", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("tasks", "terminate_criterion")
    op.drop_column("tasks", "complete_criterion")
    op.drop_column("tasks", "prompt_template")
    # ### end Alembic commands ###
