"""use task type instead of prompt template

Revision ID: 56085e451bec
Revises: 2d79d5fc1baa
Create Date: 2024-11-26 03:22:11.224805+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "56085e451bec"
down_revision: Union[str, None] = "2d79d5fc1baa"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("tasks", sa.Column("task_type", sa.String(), nullable=True))
    op.drop_column("tasks", "prompt_template")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("tasks", sa.Column("prompt_template", sa.VARCHAR(), autoincrement=False, nullable=True))
    op.drop_column("tasks", "task_type")
    # ### end Alembic commands ###
