You will be given information about a user's goal and details. 

Your job is to answer the user's questions based on the information provided.

The user's questions will be provided in JSON format.

Your answers should be direct and to the point. No need to explain the answer.

Your response should be in JSON format. Basically fill in the answer part and return the JSON.

User's goal: {{ navigation_goal }}

User's details: {{ navigation_payload }}

User's questions: {{ queries_and_answers }}

YOUR RESPONSE HAS TO BE IN JSON FORMAT. DO NOT RETURN ANYTHING ELSE. 
THESE ANSWERS WILL BE USED TO FILL OUT INFORMATION ON A WEBPAGE. DO NOT INCLUDE ANY UNRELATED INFORMATION OR UNNECESSARY DETAILS IN YOUR ANSWERS.

EXAMPLE RESPONSE FORMAT:
{
  "question_1": "answer_1",
  "question_2": "answer_2",
  "question_3": "answer_3"
}