"""observer summary and output

Revision ID: 6a947c379c02
Revises: d5640aa644b9
Create Date: 2025-01-10 22:46:41.757862+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "6a947c379c02"
down_revision: Union[str, None] = "d5640aa644b9"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("summary", sa.String(), nullable=True))
    op.add_column("observer_cruises", sa.Column("output", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_cruises", "output")
    op.drop_column("observer_cruises", "summary")
    # ### end Alembic commands ###
