"""add error_code_mapping and extracted_information_schema to task v2

Revision ID: c6c0eee7f88d
Revises: 268dcc995513
Create Date: 2025-03-18 18:50:28.930836+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "c6c0eee7f88d"
down_revision: Union[str, None] = "268dcc995513"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("extracted_information_schema", sa.JSON(), nullable=True))
    op.add_column("observer_cruises", sa.Column("error_code_mapping", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_cruises", "error_code_mapping")
    op.drop_column("observer_cruises", "extracted_information_schema")
    # ### end Alembic commands ###
