"""add created_at and modified_at to observer tables;

Revision ID: c502ecf908c6
Revises: dc2a8facf0d7
Create Date: 2024-12-09 00:40:30.098534+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "c502ecf908c6"
down_revision: Union[str, None] = "dc2a8facf0d7"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("created_at", sa.DateTime(), nullable=False))
    op.add_column("observer_cruises", sa.Column("modified_at", sa.DateTime(), nullable=False))
    op.add_column("observer_thoughts", sa.Column("created_at", sa.DateTime(), nullable=False))
    op.add_column("observer_thoughts", sa.Column("modified_at", sa.DateTime(), nullable=False))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_thoughts", "modified_at")
    op.drop_column("observer_thoughts", "created_at")
    op.drop_column("observer_cruises", "modified_at")
    op.drop_column("observer_cruises", "created_at")
    # ### end Alembic commands ###
