"""add_RESIDENTIAL_DE_proxy

Revision ID: 5dd8928389c5
Revises: df80b5d155d0
Create Date: 2025-01-29 05:54:03.709008+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "5dd8928389c5"
down_revision: Union[str, None] = "df80b5d155d0"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute("ALTER TYPE proxylocation ADD VALUE 'RESIDENTIAL_DE'")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    # Cannot remove enum values in PostgreSQL
    pass
    # ### end Alembic commands ###
