You are given a screenshot, user data extraction goal, the JSON schema for the output data format, and the current URL.

Your task is to extract the requested information from the screenshot and {% if extracted_information_schema %}output it in the specified JSON schema format:
{{ extracted_information_schema }} {% else %}output in strictly JSON format {% endif %}

Add as much details as possible to the output JSON object while conforming to the output JSON schema.

Do not ever include anything other than the JSON object in your output, and do not ever include any additional fields in the JSON object.

If you are unable to extract the requested information for a specific field in the json schema, please output a null value for that field.

If you are trying to extract the href links which are using the jinja style like "{% raw %}{{}}{% endraw %}", please keep the orignal string.

User Data Extraction Goal: {{ data_extraction_goal }}

{% if previous_extracted_information %}
Previous contexts or thoughts: ```{{ previous_extracted_information }}```
{% endif %}

{% if error_code_mapping_str %}
Use the error codes and their descriptions to return errors in the output, do not return any error that's not defined by the user. Don't return any outputs if the schema doesn't specify an error related field. Here are the descriptions defined by the user: {{ error_code_mapping_str }}
{% endif %}

Clickable elements from `{{ current_url }}`:
```
{{ elements }}
```

Current URL: {{ current_url }}

Text extracted from the webpage: {{ extracted_text }}

User Navigation Payload: {{ navigation_payload }}

Current datetime, ISO format:
```
{{ local_datetime }}
```