"""add current_value, current_index, loop_values to workflow_run_blocks table

Revision ID: cf3cd8d666b0
Revises: 5be249d8dc96
Create Date: 2024-12-23 09:07:57.592369+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "cf3cd8d666b0"
down_revision: Union[str, None] = "5be249d8dc96"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_run_blocks", sa.Column("loop_values", sa.JSON(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("current_value", sa.String(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("current_index", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_run_blocks", "current_index")
    op.drop_column("workflow_run_blocks", "current_value")
    op.drop_column("workflow_run_blocks", "loop_values")
    # ### end Alembic commands ###
