"""add workflow_permanent_id to observer_cruises and observer_thoughts

Revision ID: 411dd89f3df9
Revises: c502ecf908c6
Create Date: 2024-12-16 22:20:52.174896+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "411dd89f3df9"
down_revision: Union[str, None] = "c502ecf908c6"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("workflow_permanent_id", sa.String(), nullable=True))
    op.add_column("observer_thoughts", sa.Column("workflow_permanent_id", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_thoughts", "workflow_permanent_id")
    op.drop_column("observer_cruises", "workflow_permanent_id")
    # ### end Alembic commands ###
