You are given a svg element. You need to figure out what its shape means.
SVG Element:
```
{{svg_element}}
```

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.
Reply in JSON format with the following keys:
{
    "confidence_float": float, // The confidence of the action. Pick a number between 0.0 and 1.0. 0.0 means no confidence, 1.0 means full confidence
    "shape": string, // A short description of the shape of SVG and its meaning
    "recognized": bool, // False if you can't recognize the shape or you don't understand the meaning, otherwise true.
}