The user is trying to achieve a goal in the browser assisted by a browser AI assistant.

According to the AI assistant's feedback, including reasoning and its message, there might be a question to answer or a decision to be made.

Help answer the question or make the decision based on the information provided. Try your best to help the user achieve the goal.

Reply in JSON format with the following keys:
{
    "question_or_decision": str // What is the question to answer or what is the decision to make?
    "thought": str // Think step by step. What kind of feedback can best help the AI assistant move forward and achieve the goal?
    "enough_information": bool // Whether the information provided is enough to answer the question or make the decision.
    "answer": str // The answer to the question or the decision to make. Give the answer as if you're talking to the assistant.
}

User goal: {{ navigation_goal }}

Assistant reasoning: {{ assistant_reasoning }}

Assistant message: {{ assistant_message }}