"""new observer thoughts

Revision ID: d13af1e466fa
Revises: 835522a23b19
Create Date: 2024-12-27 16:10:36.555540+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d13af1e466fa"
down_revision: Union[str, None] = "835522a23b19"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_thoughts", sa.Column("observer_thought_type", sa.String(), nullable=True))
    op.add_column("observer_thoughts", sa.Column("observer_thought_scenario", sa.String(), nullable=True))
    op.add_column("observer_thoughts", sa.Column("output", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_thoughts", "output")
    op.drop_column("observer_thoughts", "observer_thought_scenario")
    op.drop_column("observer_thoughts", "observer_thought_type")
    # ### end Alembic commands ###
