"""Add collection id to bitwarden credential parameters

Revision ID: 2c163e606a3d
Revises: 312d305c6b18
Create Date: 2024-06-11 05:02:25.023252+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "2c163e606a3d"
down_revision: Union[str, None] = "312d305c6b18"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "bitwarden_login_credential_parameters", sa.Column("bitwarden_collection_id", sa.String(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("bitwarden_login_credential_parameters", "bitwarden_collection_id")
    # ### end Alembic commands ###
